// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Omnibus message base
///
/// An omnibus message has an [omnibusId] (statically defined per message
/// implementation as [Id]) so it can be safely identified by as a separate
/// message in unchecked mode.
abstract class _OmnibusMessage {
  /// Message identifier key
  ///
  /// Use the following convention for this identifier:
  /// `package_name.ClassName`
  static const String Id = 'omnibus.OmnibusPublicMessage';

  /// Getter for request identifier
  String get omnibusId => Id;
}

/// Public message
///
/// A public message is a message that is broadcasted through the message bus to
/// all connected handlers. It contains a [getConditions] method to generate a
/// condition map. Handlers can select which messages they want to receive by
/// specifying a conditions map as well.
class _OmnibusPublicMessage extends _OmnibusMessage {
  /// Generate condition map
  ///
  /// This method should return a condition map that is generated from the class
  /// fields. The condition map contains a series of values that describe some
  /// details about this message that can be used to filter messages. An example
  /// of this is an `OnClickEvent` that specifies the ID of the clicked element.
  /// Listeners can bind to a specific request or event message and specify
  /// which message they want to receive using their own condition map with the
  /// same keys as used by the message class. The Omnibus will only trigger the
  /// handler if the condition maps of the message and the handler match.
  Map<String, dynamic> getConditions() {
    return {};
  }
}

/// Event message
///
/// An event message is emitted when the publisher wants to notify other
/// handlers of what is happening. An example of this is a progress update. This
/// message can be used for publish-subscribe
/// messasing.
abstract class OmnibusEvent extends _OmnibusPublicMessage {
  static const String ID = 'omnibus.OmnibusEvent';
  String get omnibusId => ID;
}

/// Request message
///
/// A request message is emitted when the publisher needs certain data. An
/// example of this is a lifecycle event that notifies all handlers that the
/// application is going to shut down and requests all handlers to reply the
/// user data they want to store. This message can be used for request-reply
/// messasing.
abstract class OmnibusRequest extends _OmnibusPublicMessage {
  static const String ID = 'omnibus.OmnibusRequest';
  String get omnibusId => ID;
}

/// Reply message
///
/// The reply message is emitted when the publisher wants to reply to a request
/// message.
abstract class OmnibusResponse extends _OmnibusMessage {}
